/*
 * Code and supporting documentation (c) Copyright 1990 1991 Tektronix, Inc.
 * 	All Rights Reserved
 * 
 * This file is a component of an X Window System client which uses the Xcms 
 * Color Management System.  TekColor is a trademark of Tektronix, Inc.  The
 * TekColor Editor is the subject of U.S. and foreign patents pending.  The
 * term "TekHVC" designates a particular color space that is the subject of
 * U.S. Patent No. 4,985,853 (equivalent foreign patents pending).
 * Permission is hereby granted to use, copy, modify, sell, and otherwise
 * distribute this software and its documentation for the X Window System
 * environment, for any purpose and without fee, provided that:
 * 
 * 1.	The code and documentation are only used to implement a 
 *      TekColor Editor in an X Window System environment; and
 * 2.	This copyright and permission notice is reproduced in all copies
 * 	of the code and in supporting documentation.
 * 
 * Permission is granted to modify this code as required to allow it to
 * be compiled on any host computer, provided that the functionality of
 * the TekColor Editor is not modified in any way.  A description of any 
 * modifications must be sent to Tektronix, Inc.  Contact 
 * Tektronix Inc., P.O. Box 1000, Mail Station 60-850, 
 * Network Displays Division Engineering, Wilsonville, OR 97070.
 *
 * Tektronix makes no representation about the suitability of this software
 * for any purpose.  It is provided "as is" and with all faults.
 * 
 * TEKTRONIX DISCLAIMS ALL WARRANTIES APPLICABLE TO THIS SOFTWARE,
 * INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT SHALL TEKTRONIX BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA, OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR THE PERFORMANCE OF THIS SOFTWARE.
 * 
 *	NAME
 *		menu - create and manage menus
 *
 *	DESCRIPTION
 *		Create the menubar, all pulldown and cascading menus,
 *		buttons and dialogs, including help stuff.  Manage callbacks
 *		invoked by menu choices.
 */
#ifndef LINT
static char *copy_notice = "Copyright 1991 Tektronix, Inc.";
#ifdef RCS_ID
static char *rcsid=  "$Header: menu.c,v 1.3 91/08/28 10:02:26 adamsc Exp $";
#endif /* RCS_ID */
#endif /* LINT */

/*
 *      EXTERNAL INCLUDES
 */


/*
 *      INTERNAL INCLUDES
 */
#include "xtici.h"
#include "constrain.off.bm"
#include "constrain.on.bm"

#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xmu/Misc.h>
#include "widgets/xticiSD.h"
#include "widgets/ColorSP.h"

/*
 *      EXTERNS
 */

/*
 *      GLOBALS
 */
#ifdef DEC
#  define GLOBAL	global
#else
#  define GLOBAL
#endif

/* controls how/if chromas are restricted */
GLOBAL int clampState = ClampMonitor;

/*
 *      LOCAL DEFINES
 */
#define EditCopy	0
#define EditPaste	1

#define EditOne		0

#define CopyHVC		0
#define CopyRGB	    	1
#define CopyuvY		2

#define OptionHue	0
#define OptionLeaf	1
#define OptionClamp	2

#define ViewCoord	0

#define ViewRGB		0
#define ViewUVY		1

#define ImportPixel	0
#define ExportPixel	1

#define HelpInterface	0
#define HelpVersion	1
#define HelpQuit	2
#define HelpEdit	3
#define HelpOptions	4
#define HelpImport	5
#define HelpExport	6

/*
 *      LOCAL TYPEDEFS
 */


/*
 *      LOCALS VARIABLES
 */
#ifdef DEBUG
#  define STATIC
#else
#  define STATIC        static
#endif

/* These are a series of popup menus that are needed */
STATIC Widget coordmenu, copymenu, editmenu, helpmenu, optionmenu;

/* These widgets control which coordinate sets are viewed */
STATIC Widget rgbCoord, uvyCoord;

/* These widgets are help dialogs */
STATIC Widget ifHelp, versHelp, quitHelp;
STATIC Widget editHelp, importHelp, optionHelp, exportHelp;

/* These are the routines that create them */
STATIC void CreateHelpIf();
STATIC void CreateHelpVersion();
STATIC void CreateHelpQuit();
STATIC void CreateHelpEdit();
STATIC void CreateHelpOptions();
STATIC void CreateHelpImport();
STATIC void CreateHelpExport();
STATIC void PlaceDropDownMenu();
STATIC void PlaceCascadeMenu();

STATIC char *exitHelp = "\
You have made some changes in the current colormap.  Those changes will not\n\
be saved automatically when the program exits.  If you want to retain your\n\
changes, cancel this operation and explicitly save the changed colormap.";

STATIC char *nograbHelp = "\
In X, normally all pointer actions go to the window where the action occurs.\n\
The editor \"grabs\" the pointer so that it can be used to identify the \n\
object to act on from any window, and prevent any other window from seeing\n\
that action.\n\
This message was generated because the attempt to grab the pointer failed.\n\
Normally this happens because some other client has already grabbed it.\n\
Try to import again.  If you continue to see this message, try to determine\n\
which client in your environment (maybe the window manager) has the pointer\n\
grabbed.";


/************************************************************************
 *									*
 *			PRIVATE ROUTINES				*
 *									*
 ************************************************************************/
/* ARGSUSED */
STATIC void Quit(w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    if (EditedMap(mapSize) && 
	(Warning("Upon exit, current colormap changes will not be saved", 
		    exitHelp) == MessageCancel))
	return;
    exit(0);
}

/* ARGSUSED */
STATIC void HelpMenuPopup (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int which = (int)pwhich;
    Widget help = NULL;

    XtPopdown (helpmenu);

    switch (which) {
    case HelpInterface:
	if (!ifHelp)
	    CreateHelpIf(w);
	help = ifHelp;
	break;
    case HelpVersion:
	if (!versHelp)
	    CreateHelpVersion(w);
	help = versHelp;
	break;
    case HelpQuit:
	if (!quitHelp)
	    CreateHelpQuit(w);
	help = quitHelp;
	break;
    case HelpEdit:
	if (!editHelp)
	    CreateHelpEdit(w);
	help = editHelp;
	break;
    case HelpOptions:
	if (!optionHelp)
	    CreateHelpOptions(w);
	help = optionHelp;
	break;
    case HelpImport:
	if (!importHelp)
	    CreateHelpImport(w);
	help = importHelp;
	break;
    case HelpExport:
	if (!exportHelp)
	    CreateHelpExport(w);
	help = exportHelp;
	break;
     }

    if (help) {
	XtPopup (help, XtGrabNonexclusive);
    }
}

STATIC void NewClampState(new, what)
int new;
int what;
{
    Pixel *indexes;
    int i, ct;
    XcmsColor color;

    clampState = new;

    /* apply new clamping to current colors */
    indexes = EditingIndexes(&ct);
    for (i = 0; i < ct; i++)
	ConstrainColor(indexes[i]);
    bcopy ((char *)&currentMap[currentIndex],(char *)&color,sizeof(XcmsColor));
    NewColor(&color, what, ChromaChange);
}

STATIC Pixmap bitmapOn, bitmapOff;  /* This is the bitmaps for clamp widget */

/* ARGSUSED */
STATIC void OptionsClamp(w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    Arg args[1];
    Pixel fore, back;
    int state = (clampState == ClampNone ? ClampMonitor : ClampNone);

    XtPopdown (optionmenu);
    NewClampState(state, MenuWidget);

    /* Update the widget to reflect new state by the foreground */
    /* and background of the x */
    if (state == ClampNone) 
	XtSetArg(args[0], XtNbitmap, bitmapOff);
    else
	XtSetArg(args[0], XtNbitmap, bitmapOn);
    XtSetValues(w, args, 1);
}

/* ARGSUSED */
STATIC void OptionsLeaf (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int state = (leafState == LeafEmpty ? LeafFilled : LeafEmpty);

    XtPopdown (optionmenu);
    NewLeafState(state, MenuWidget);
}

/* ARGSUSED */
STATIC void OptionsHue (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int state = (huebarState == HuebarEmpty ? HuebarConstant : HuebarEmpty);

    XtPopdown (optionmenu);
    NewHuebarState(state, MenuWidget);
}

/*
 * Coordinate displays are attached right-to-left, in the following order:
 *  (patch) - u'v'Y - rgb - HVC - (form).
 */
/* ARGSUSED */
void Coordinates (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int which = (int)pwhich;
    int target;
    void NewShow();

    XtPopdown (coordmenu);
    XtPopdown (optionmenu);

    switch (which) {
    case ViewRGB:
	target = RgbWidget;
	break;
    case ViewUVY:
	target = UvyWidget;
	break;
    }

    NewShow(target);
}

extern void HandleColorCopy();

/* ARGSUSED */
void Copy (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int which = (int)pwhich;
    unsigned int format;

    XtPopdown (copymenu);
    XtPopdown (editmenu);

    /* Must convert from HVC to what ever color is asked for here */
    bcopy ((char *)&currentHvc, (char *)&clipColor, sizeof (XcmsColor));
    switch (which) {
	case CopyHVC:
	    format = XcmsTekHVCFormat;
	    break;
	case CopyRGB:
	    format = XcmsRGBFormat;
	    break;
	case CopyuvY:
	    format = XcmsCIEuvYFormat;
	    break;
	default:
	    return;
    }
    if ((XcmsConvertColors (pDefaultCCC, &clipColor, 1, format, NULL))
	== XcmsFailure) {
	if (Warning("Could not convert to selected format", (char *)NULL)
	    != MessageContinue) {
	    return;
	} 
	bcopy ((char *)&currentHvc, (char *)&clipColor, sizeof (XcmsColor));
    }
    HandleColorCopy ();
}

/* ARGSUSED */
void Grab (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    int which = (int)pwhich;
    Window win = XtWindow(primary);

    /* Just use any visible widget for the grab window */
    if (XGrabPointer(dpy, win, False, 
			ButtonPressMask | ButtonReleaseMask,
			GrabModeAsync, GrabModeAsync, 
			None, grabCursor, CurrentTime)) {
	InformMsg("Could not grab pointer", nograbHelp);
	return;
    }

    switch (which) {
    case ImportPixel:
	grabState = GrabUp | GrabPixel;
	break;
    case ExportPixel:
	/* take the selection now, so it can be later exported */
	if (!selectionMine)
	    TakeSelection();
	grabState = GrabUp | GrabCell;
	break;
    }
}

extern int HandleColorPaste ();

/* ARGSUSED */
void Paste (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    XtPopdown (editmenu);
    /* get the pixel value of the current pixel in the patch */
    clipColor.pixel = currentHvc.pixel;
    HandleColorPaste (&clipColor);
}

/* ARGSUSED */
STATIC void KillWidget (w, pwhich, call)
Widget w;
XtPointer pwhich;
XtPointer call;
{
    XtPopdown ((Widget)pwhich);
}

/*ARGSUSED*/
STATIC Widget CreateHelp(parent, title, message)
Widget parent;
String title;
String message;
{
    Arg args[5];
    int argCt;
    Widget shell, form, help, ok;

    argCt = 0;
    shell = XtCreatePopupShell("help_popup", transientShellWidgetClass, 
				  primary, args, argCt);

    form = XtCreateManagedWidget("form", formWidgetClass, shell, NULL, 0);
    
    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, message);		argCt++;
    XtSetArg(args[argCt], XtNjustify, XtJustifyLeft);	argCt++;
    help = XtCreateManagedWidget(title, labelWidgetClass, form, args, argCt);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "OK");		argCt++;
    XtSetArg(args[argCt], XtNjustify, XtJustifyCenter);	argCt++;
    XtSetArg(args[argCt], XtNfromVert, help);		argCt++;
    ok = XtCreateManagedWidget(title, commandWidgetClass, form, args, argCt);
    XtAddCallback (ok, XtNcallback, KillWidget, (XtPointer)shell);

    return(shell);
}

STATIC void CreateHelpIf(parent)
Widget parent;
{
    char *str = "\
First choose a colormap entry to edit using either the\n\
colormap control or by importing a pixel from another window.\n\
Then modify hue using the vertical hue bar.  \n\
Modify value and chroma using the leaf\n\
to the right of the hue bar.\n\
\n\
Hue describes the color family, e.g., red, yellow, magenta.\n\
It is a polar coordinate ranging from 0 (red) to 360 degrees.\n\
\n\
Value describes the lightness of the color,\n\
ranging from 0 (black) to 100 (white).\n\
\n\
Chroma describes the vividness of the color.\n\
The maximum chroma achievable for a given hue and value varies,\n\
but 0 (gray) is always the minimum chroma.";

    ifHelp = CreateHelp(parent, "color editor interface help", str);
}

STATIC void CreateHelpEdit(parent)
Widget parent;
{
    char *str = "\
Copy color copies the color description of the current color cell into a \n\
public selection.  The use of the ICCCM selection mechanism for PRIMARY and\n\
STRING allows this function to be used in copying and pasting color values.\n\
\n\
Paste copies the color description in the public selection \n\
into the current color cell.  The use of the ICCCM selection mechanism \n\
for PRIMARY and STRING allows this function selecting color values.";

    editHelp = CreateHelp(parent, "color editor Edit menu help", str);
}

STATIC void CreateHelpExport(parent)
Widget parent;
{
    char *str = "\
Export the current color to another window.\n\
\n\
In X, it is not possible for the editor to directly modify a color of a\n\
different window.  For the color to be exported, the program managing that\n\
window must cooperate with the editor to change the color.  If the editor\n\
detects that the client will not accept the requested change, a dialog\n\
will appear explaining that the color cannot be exported.  Even if the\n\
client accepts the new color from the editor, it is up to the client as to\n\
how the new color will be used.";

    exportHelp = CreateHelp(parent, "color editor Export menu help", str);
}

STATIC void CreateHelpQuit(parent)
Widget parent;
{
    char *str = "Quit terminates the editor.";

    quitHelp = CreateHelp(parent, "color editor File menu help", str);
}

STATIC void CreateHelpImport(parent)
Widget parent;
{
    char *str = "\
Import the color associated with a pixel from another window.\n\
Any changes to that color cell will be lost.\n\
When this option is chosen, the editor goes into a mode that allows\n\
mouse input in any window to go to the editor - a grab.  Move the cursor\n\
to any pixel on the screen and click the selection button.\n\
The color cell being edited changes to match the chosen pixel.";

    importHelp = CreateHelp(parent, "color editor Import menu help", str);
}

STATIC void CreateHelpOptions(parent)
Widget parent;
{
    char *str = "\
Fill hue bar controls the appearance of the hue bar.\n\
Empty fills the bar in the background color.\n\
Constant fills the bar with a sample of colors of different hue.\n\
Each color is that with the maximum chroma for that hue.\n\
The constant option uses colors from the colormap,\n\
so editing conflicts are possible when these options are displayed.\n\
\n\
Fill leaf with a sample of colors of constant hue, varying value\n\
and chroma.  This is only a portion of the colors available for this hue.\n\
Colors from the colormap are used to fill the leaf, so editing conflicts\n\
are possible when this option is chosen.\n\
\n\
Coordinates specifies which coordinate systems are used to display\n\
a description of the current color.\n\
\n\
Constrain chroma controls what happens when a color with an unreachable\n\
chroma is specified.  None allows any chroma up to 120.0.  The color\n\
patch displays the closest reachable color with that hue and value.\n\
Monitor restricts colors to those that can be generated on the terminal\n\
or workstation display.";

    optionHelp = CreateHelp(parent, "color editor options help", str);
}

STATIC void CreateHelpVersion(parent)
Widget parent;
{
    char *str = "\
Tektronix Color Editor Version 1.0b (X - Athena Widgets)\n\
\n\
This is an X Window System client version of the TekColor interface.\n\
\n\
Copyright Tektronix, Inc., 1989, 1990, 1991\n\
Patent pending for portions of the TekColor system.\n\
\n\
TekColor and TekHVC are trademarks of Tektronix, Inc.\n\
X Window System is a trademark of the Massachusetts Institute of Technology\n\
Motif is a trademark of Open Software Foundation, Inc.";

    versHelp = CreateHelp(parent, "color editor version help", str);
}


/*
 * Create the Edit pulldown menu and sub-menus.
 */
STATIC Arg editargs[] = {
    {XtNfromHoriz, NULL},
    {XtNlabel, (XtArgVal)"Edit"}
};

STATIC Widget CreateEditPulldown(parent, neighbor)
Widget parent, neighbor;
{
    Widget edit, menuform, copyform;
    Widget copy, paste;		/* menu entries */
    Widget one, two, three;	/* sub menu entries for copy command */
    Arg args[5];
    int argCt;

    /* this is the button on the menubar */
    editargs[0].value = (XtArgVal)neighbor;
    edit = XtCreateManagedWidget("edit", commandWidgetClass, parent,
					editargs, XtNumber(editargs));

    /* create the editmenu popup */
    argCt = 0;
    XtSetArg(args[argCt], XtNoverrideRedirect, True);       argCt++;
    editmenu = XtCreatePopupShell("editmenu", transientShellWidgetClass, edit, 
				  args, argCt);
    XtAddCallback(edit, XtNcallback, PlaceDropDownMenu, (XtPointer)editmenu);

    menuform = XtCreateManagedWidget("menuform", formWidgetClass,
				    editmenu, NULL, 0);

    /* Create the sub menu used by Copy color */
    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "Copy Color ->");		argCt++;
    copy = XtCreateManagedWidget("copy", commandWidgetClass, menuform,
				    args, argCt);
    argCt = 0;
    XtSetArg(args[argCt], XtNoverrideRedirect, True);       argCt++;
    copymenu = XtCreatePopupShell("copymenu", transientShellWidgetClass,
				  copy, args, argCt);
    XtAddCallback(copy, XtNcallback, PlaceCascadeMenu, (XtPointer)copymenu);

    copyform = XtCreateManagedWidget("menuform", formWidgetClass,
				    copymenu, NULL, 0);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "TekHVC");			argCt++;
    one =  XtCreateManagedWidget("copyhvc", commandWidgetClass, copyform,
				    args, argCt);
    XtAddCallback(one, XtNcallback, Copy, (XtPointer)CopyHVC);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "RGB");			argCt++;
    XtSetArg(args[argCt], XtNfromVert, one);			argCt++;
    two =  XtCreateManagedWidget("copyrgb", commandWidgetClass, copyform,
				    args, argCt);
    XtAddCallback(two, XtNcallback, Copy, (XtPointer)CopyRGB);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "CIE u'v'Y");		argCt++;
    XtSetArg(args[argCt], XtNfromVert, two);			argCt++;
    three =  XtCreateManagedWidget("copyuvy", commandWidgetClass, copyform,
				    args, argCt);
    XtAddCallback(three, XtNcallback, Copy, (XtPointer)CopyuvY);

    /* Create the Paste Button on the Edit Menu */
    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "Paste Color");		argCt++;
    XtSetArg(args[argCt], XtNfromVert, copy);			argCt++;
    paste = XtCreateManagedWidget("paste", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(paste, XtNcallback, Paste, (XtPointer)EditPaste);

    return(edit);
}


/*
 * Create the Help pulldown menu; dialogs are created on demand
 */
STATIC Arg helpargs[] = {
    {XtNfromHoriz, NULL},
    {XtNlabel, (XtArgVal)"Help"},
};

STATIC void CreateHelpPulldown(parent, neighbor)
Widget parent, neighbor;
{
    Widget help, menuform, interf, version, quit, edit, option, import, export;
    Arg args[5];
    int argCt;

    /* create the button to go in the menu bar */
    helpargs[0].value = (XtArgVal)neighbor;
    help = XtCreateManagedWidget("help", commandWidgetClass, parent, 
					helpargs, XtNumber(helpargs));

    /* create the pulldown menu */
    argCt = 0;
    XtSetArg(args[argCt], XtNoverrideRedirect, True);	    argCt++;
    helpmenu = XtCreatePopupShell ("helpmenu", transientShellWidgetClass,
				       help, args, argCt);
    XtAddCallback(help, XtNcallback, PlaceDropDownMenu, (XtPointer)helpmenu);

    menuform = XtCreateManagedWidget("menuform", formWidgetClass,
				    helpmenu, NULL, 0);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Interface");	    argCt++;
    interf = XtCreateManagedWidget("interface", commandWidgetClass, menuform,
				   args, argCt);
    XtAddCallback(interf, XtNcallback, HelpMenuPopup, HelpInterface);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Version");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, interf);		    argCt++;
    version = XtCreateManagedWidget("version", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(version, XtNcallback, HelpMenuPopup, (XtPointer)HelpVersion);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Quit Menu");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, version);	    argCt++;
    quit = XtCreateManagedWidget("quit", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(quit, XtNcallback, HelpMenuPopup, (XtPointer)HelpQuit);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Options Menu");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, quit);		    argCt++;
    option = XtCreateManagedWidget("option", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(option, XtNcallback, HelpMenuPopup, (XtPointer)HelpOptions);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Import Menu");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, option);		    argCt++;
    import = XtCreateManagedWidget("import", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(import, XtNcallback, HelpMenuPopup, (XtPointer)HelpImport);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Export Menu");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, import);		    argCt++;
    export = XtCreateManagedWidget("export", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(export, XtNcallback, HelpMenuPopup, (XtPointer)HelpExport);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "On Edit Menu");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, export);		    argCt++;
    edit = XtCreateManagedWidget("edit", commandWidgetClass, menuform,
				    args, argCt);
    XtAddCallback(edit, XtNcallback, HelpMenuPopup, (XtPointer)HelpEdit);
}



/*
 * Create the Option pulldown menu button and tree
 */
STATIC Arg optionargs[] = {
    {XtNfromHoriz, NULL},
    {XtNlabel, (XtArgVal)"Options" }
};
STATIC Arg hfbargs[] = {
    {XtNlabel, (XtArgVal)"Fill Hue Bar"},
};
STATIC Arg lfbargs[] = {
    {XtNlabel, (XtArgVal)"Fill Leaf"},
    {XtNfromVert, (XtArgVal)NULL},
};
STATIC Arg clbargs[] = {
    {XtNbitmap, (XtArgVal)NULL},  
    /* or use {XtNlabel, (XtArgVal)"Constrain Chroma"}, */
    {XtNfromVert, (XtArgVal)NULL},
};
STATIC Arg cobargs[] = {
    {XtNlabel, (XtArgVal)"Coodinates ->"},
    {XtNfromVert, (XtArgVal)NULL},
};

STATIC Widget CreateOptionPulldown(parent, neighbor)
Widget parent, neighbor;
{
    Widget option, menuform, coordform;
    /* menu entries */
    Widget huefillbttn;
    Widget leaffillbttn;
    Widget clampbttn;
    Widget coordbttn;
    Widget rgb, uvy;		    /* coordinates cascade */
    Arg  args[5];
    int  argCt;

    /* the menu item in the menubar */
    optionargs[0].value = (XtArgVal)neighbor;
    option = XtCreateManagedWidget("option", commandWidgetClass,
				    parent, optionargs, XtNumber(optionargs));

    /* Create each of the sub menu items and the sub menu cascades, etc. */
    argCt = 0;
    XtSetArg(args[argCt], XtNoverrideRedirect, True);       argCt++;
    optionmenu = XtCreatePopupShell("optionmenu", transientShellWidgetClass,
				    option, args, argCt);
    XtAddCallback(option, XtNcallback, PlaceDropDownMenu, (XtPointer)optionmenu);

    menuform = XtCreateManagedWidget("menuform", formWidgetClass,
				    optionmenu, NULL, 0);

    /* Create the menu entries in the optionmenu pulldown */

    /* only allow hue optionmenu of colormap is large enough */
    if (allowFill <= FillHuebar) {
	/* Create the Fill Hue Bar command widget */
	huefillbttn = XtCreateManagedWidget("huebutton", commandWidgetClass, 
				    menuform, hfbargs, XtNumber(hfbargs));
	XtAddCallback(huefillbttn, XtNcallback, OptionsHue, (XtPointer)NULL);

	/* Create the Fill Leaf command widget */
	lfbargs[1].value = (XtArgVal)huefillbttn;
	leaffillbttn = XtCreateManagedWidget("leafbutton", commandWidgetClass,
				    menuform, lfbargs, XtNumber(lfbargs));
	XtAddCallback(leaffillbttn, XtNcallback, OptionsLeaf, (XtPointer)NULL);
	clbargs[1].value = (XtArgVal)leaffillbttn;
    }

    /* Create the Clamp command widget */
    bitmapOff = XCreateBitmapFromData(dpy, RootWindow(dpy, DefaultScreen(dpy)),
				      constrain_off_bits, 
				      constrain_width, constrain_height);
    bitmapOn = XCreateBitmapFromData(dpy, RootWindow(dpy, DefaultScreen(dpy)),
				     constrain_on_bits,
				     constrain_width, constrain_height);
    clbargs[0].value = (XtArgVal)bitmapOn;
    clampbttn = XtCreateManagedWidget("clamp", commandWidgetClass, menuform,
				      clbargs, XtNumber(clbargs));
    XtAddCallback(clampbttn, XtNcallback, OptionsClamp, (XtPointer)NULL);

    /* Create coordinates button and pulldown */
    cobargs[1].value = (XtArgVal)clampbttn;
    coordbttn = XtCreateManagedWidget("coordinates", commandWidgetClass,
				    menuform, cobargs, XtNumber(cobargs));

    argCt = 0;
    XtSetArg(args[argCt], XtNoverrideRedirect, True);       argCt++;
    coordmenu = XtCreatePopupShell("coordmenu", transientShellWidgetClass, 
				    coordbttn, args, argCt);
    XtAddCallback(coordbttn, XtNcallback, PlaceCascadeMenu, (XtPointer)coordmenu);

    coordform = XtCreateManagedWidget("menuform", formWidgetClass,
				     coordmenu, NULL, 0);

    /* Create the coordinates buttons */
    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "RGB");		    argCt++;
    rgb = XtCreateManagedWidget("rgb", commandWidgetClass, coordform,
				args, argCt);
    rgbCoord = rgb;
    XtAddCallback(rgb, XtNcallback, Coordinates, (XtPointer)ViewRGB);

    argCt = 0;
    XtSetArg(args[argCt], XtNlabel, "CIE u'v'Y");	    argCt++;
    XtSetArg(args[argCt], XtNfromVert, rgbCoord);	    argCt++;
    uvy = XtCreateManagedWidget("uvY", commandWidgetClass, coordform,
				args, argCt);
    uvyCoord = uvy;
    XtAddCallback(uvy, XtNcallback, Coordinates, (XtPointer)ViewUVY);

    return(option);
}


/************************************************************************
 *									*
 *			PUBLIC ROUTINES					*
 *									*
 ************************************************************************/
STATIC Arg quitargs[] = {
    {XtNlabel, (XtArgVal)"Quit"},
};
STATIC Arg importargs[] = {
    {XtNfromHoriz, NULL},
    {XtNlabel, (XtArgVal)"Import"},
};
STATIC Arg exportargs[] = {
    {XtNfromHoriz, NULL},
    {XtNlabel, (XtArgVal)"Export"},
};

/*
 * Create a menu bar widget, its pulldown menus and all buttons, dialogs,
 * etc. found in the menu hierarchy.
 */
Widget CreateMenuBar(parent)
Widget parent;
{
    Widget self = XtCreateManagedWidget("menubar", formWidgetClass, parent, 
					NULL, 0);
    Widget quit, edit, option, import, export;

    quit = XtCreateManagedWidget("quit", commandWidgetClass, self,
				 quitargs, XtNumber(quitargs));
    XtAddCallback(quit, XtNcallback, Quit, (XtPointer)NULL);

    option = CreateOptionPulldown(self, quit);

    importargs[0].value = (XtArgVal)option;
    import = XtCreateManagedWidget("import", commandWidgetClass, self,
				    importargs, XtNumber(importargs));
    XtAddCallback(import, XtNcallback, Grab, (XtPointer)ImportPixel);

    exportargs[0].value = (XtArgVal)import;
    export = XtCreateManagedWidget("export", commandWidgetClass, self,
				    exportargs, XtNumber(exportargs));
    XtAddCallback(export, XtNcallback, Grab, (XtPointer)ExportPixel);

    edit = CreateEditPulldown(self, export);

    CreateHelpPulldown(self, edit);

    return(self);
}

/* ARGSUSED */
void NewHuebarState(new, what)
int new;
int what;
{
    Pixel base;
    int len;
    Arg args[2];

    if (new == huebarState)
	return;

    /* make the necessary changes in the hue bar */
    switch (new) {
    case HuebarEmpty:
	HuebarRangeRestore((Pixel *)&base, (int *)&len);
	break;
    case HuebarConstant:
	if (HuebarRangeBusy()) {	    /* allow user to back out */
	    return;
	}
	if (huebarState == HuebarEmpty)
	    HuebarRangeSave();
	HuebarRangeLoad((Pixel *)&base, (int *)&len);
	break;
    }

    if (len > 0) {
	XtSetArg(args[0], XtNbase, base);
	XtSetArg(args[1], XtNlength, len);
	XtSetValues(huebar, args, XtNumber(args));
    }

    /* update the menu buttons to reflect new state */
    huebarState = new;
}

/* ARGSUSED */
void NewLeafState(new, what)
int new;
int what;
{
    Pixel base;

    if (new & LeafFilled) {
	if (LeafRangeBusy()) {		/* allow user to back out */
	    return;
	}
	LeafRangeSave((Pixel *)&base);
    } else {
	LeafRangeRestore((Pixel *)&base);
    }
    SetLeafBase(base);

    leafState = new;
}

extern void ChangeShowed();

void NewShow(new)
int new;
{
    ChangeShowed(new);

    UpdateShowWidget(new);
}

/*
 * Place a drop down widget over the top of the menu item from which it
 * has orginated. 
 */
/* ARGSUSED */
STATIC void PlaceDropDownMenu(wid, pplacewidget, call)
Widget wid;
XtPointer pplacewidget;
XtPointer call;
{
    Widget placewidget = (Widget)pplacewidget;
    Position x, y;
    Arg args[2];
    int i;

    /* 
     * Translate the coordinates of the invoking widget into root coordinates
     */
    XtTranslateCoords (wid, (Position)0, wid->core.height, &x, &y);
    
    /*
     * Now move the menu (popup shell so it is covering the invoking button.
     * Assume borderWidth of 0.
     */
    i = 0;
    XtSetArg (args[i], XtNx, x);	i++;
    XtSetArg (args[i], XtNy, y);	i++;
    XtSetValues (placewidget, args, i);
    XtPopup (placewidget, XtGrabNonexclusive);
}

/*
 * Place a cascade menu widget next to the menu item from which it
 * has orginated. 
 */
/* ARGSUSED */
STATIC void PlaceCascadeMenu(wid, pplacewidget, evnt)
Widget wid;
XtPointer pplacewidget;
XtPointer evnt;
{
    Widget placewidget = (Widget)pplacewidget;
    Position x, y;
    Dimension width = wid->core.width;
    Arg args[2];
    int i;

    /* 
     * Translate the coordinates of the invoking widget into root coordinates
     */
    XtTranslateCoords (wid, (Position)0, (Position)0, &x, &y);
    
    /*
     * Now move the menu (popup shell so it is covering the invoking button.
     * Assume borderWidth of 0.
     */
    i = 0;
    XtSetArg (args[i], XtNx, x + width);    i++;
    XtSetArg (args[i], XtNy, y);	    i++;
    XtSetValues (placewidget, args, i);
    XtPopup (placewidget, XtGrabNonexclusive);
}
